@@ -992,8 +992,8 @@ int ha_myisam::optimize(THD* thd, HA_CHECK_OPT *check_opt)
   param.sort_buffer_length=  check_opt->sort_buffer_size;
   if ((error= repair(thd,param,1)) && param.retry_repair)
   {
-    sql_print_warning("Warning: Optimize table got errno %d, retrying",
-                      my_errno);
+    sql_print_warning("Warning: Optimize table got errno %d on %s.%s, retrying",
+                      my_errno, param.db_name, param.table_name);
     param.testflag&= ~T_REP_BY_SORT;
     error= repair(thd,param,1);
   }
@@ -1383,8 +1383,8 @@ int ha_myisam::enable_indexes(uint mode)
     param.tmpdir=&mysql_tmpdir_list;
     if ((error= (repair(thd,param,0) != HA_ADMIN_OK)) && param.retry_repair)
     {
-      sql_print_warning("Warning: Enabling keys got errno %d, retrying",
-                        my_errno);
+      sql_print_warning("Warning: Enabling keys got errno %d on %s.%s, retrying",
+                        my_errno, param.db_name, param.table_name);
       /* Repairing by sort failed. Now try standard repair method. */
       param.testflag&= ~(T_REP_BY_SORT | T_QUICK);
       error= (repair(thd,param,0) != HA_ADMIN_OK);
@@ -1863,9 +1863,9 @@ int ha_myisam::create(const char *name, register TABLE *table_arg,
     {
       if ((ha_create_info->key_info[i].flags & HA_FULLTEXT))
       {
-	table_arg->key_info[i].senna_flags =ha_create_info->key_info[i].senna_flags;
-	table_arg->key_info[i].senna_initial_n_segments=ha_create_info->key_info[i].senna_initial_n_segments; 
-	table_arg->key_info[i].senna_encoding=ha_create_info->key_info[i].senna_encoding;
+	table_arg->key_info[i].senna_flags = ha_create_info->key_info[i].senna_flags;
+	table_arg->key_info[i].senna_initial_n_segments = ha_create_info->key_info[i].senna_initial_n_segments; 
+	table_arg->key_info[i].senna_encoding = ha_create_info->key_info[i].senna_encoding;
       }
     }
   }
