@@ -455,6 +455,9 @@ static SYMBOL symbols[] = {
   { "SOURCE",   SYM(SOURCE_SYM)},
   { "SPATIAL",		SYM(SPATIAL_SYM)},
   { "SPECIFIC",         SYM(SPECIFIC_SYM)},
+  { "SPLIT_ALPHA",      SYM(SENNA_SPLIT_ALPHA_SYM)},
+  { "SPLIT_DIGIT",      SYM(SENNA_SPLIT_DIGIT_SYM)},
+  { "SPLIT_SYMBOL",     SYM(SENNA_SPLIT_SYMBOL_SYM)},
   { "SQL",              SYM(SQL_SYM)},
   { "SQLEXCEPTION",     SYM(SQLEXCEPTION_SYM)},
   { "SQLSTATE",         SYM(SQLSTATE_SYM)},
@@ -661,6 +664,9 @@ static SYMBOL sql_functions[] = {
   { "ISNULL",		F_SYM(FUNC_ARG1),0,CREATE_FUNC(create_func_isnull)},
   { "IS_FREE_LOCK",	F_SYM(FUNC_ARG1),0,CREATE_FUNC(create_func_is_free_lock)},
   { "IS_USED_LOCK",	F_SYM(FUNC_ARG1),0,CREATE_FUNC(create_func_is_used_lock)},
+#ifdef ENABLE_SENNA
+  { "KWIC",             SYM(SENNA_KWIC_SYM)},
+#endif /* ENABLE_SENNA */
   { "LAST_INSERT_ID",	SYM(LAST_INSERT_ID)},
   { "ISSIMPLE",         F_SYM(FUNC_ARG1),0,CREATE_FUNC_GEOM(create_func_issimple)},
   { "LAST_DAY",         F_SYM(FUNC_ARG1),0,CREATE_FUNC(create_func_last_day)},
